# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sklearn.linear_model.base
import lale.docstrings
import lale.operators

class LinearRegressionImpl():

    def __init__(self, fit_intercept=True, normalize=False, copy_X=True, n_jobs=None):
        self._hyperparams = {
            'fit_intercept': fit_intercept,
            'normalize': normalize,
            'copy_X': copy_X,
            'n_jobs': n_jobs}
        self._wrapped_model = sklearn.linear_model.base.LinearRegression(**self._hyperparams)

    def fit(self, X, y, **fit_params):
        if fit_params is None:
            self._wrapped_model.fit(X, y)
        else:
            self._wrapped_model.fit(X, y, **fit_params)
        return self

    def predict(self, X):
        return self._wrapped_model.predict(X)

_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Ordinary least squares Linear Regression.',
    'allOf': [{
        'type': 'object',
        'required': ['fit_intercept', 'normalize', 'copy_X'],
        'relevantToOptimizer': ['fit_intercept', 'normalize'],
        'additionalProperties': False,
        'properties': {
            'fit_intercept': {
                'type': 'boolean',
                'default': True,
                'description': 'whether to calculate the intercept for this model. If set'},
            'normalize': {
                'type': 'boolean',
                'default': False,
                'description': 'This parameter is ignored when ``fit_intercept`` is set to False.'},
            'copy_X': {
                'type': 'boolean',
                'default': True,
                'description': 'If True, X will be copied; else, it may be overwritten.'},
            'n_jobs': {
                'anyOf': [{
                    'type': 'integer'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'The number of jobs to use for the computation. This will only provide'},
        }}]}

_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Fit linear model.',
    'type': 'object',
    'required': ['X', 'y'],
    'properties': {
        'X': {
            'description': 'Features; the outer array is over samples.',
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            }},
        'y': {
            'anyOf': [
            {   'type': 'array',
                'items': {
                    'type': 'array',
                    'items': {
                        'type': 'number'},
                }},
            {   'type': 'array',
                'items': {
                    'type': 'number'},
            }],
            'description': "Target values. Will be cast to X's dtype if necessary"},
        'sample_weight': {
            'anyOf': [{
                'type': 'array',
                'items': {
                    'type': 'number'},
            }, {
                'enum': [None]}],
            'description': 'Individual weights for each sample'},
    },
}
_input_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predict using the linear model',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'Samples.'},
    },
}
_output_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Returns predicted values.',
    'anyOf': [
    {   'type': 'array',
        'items': {
            'type': 'array',
            'items': {
                'type': 'number'},
        }},
    {   'type': 'array',
        'items': {
            'type': 'number'}}]}

_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': """`Linear regression`_ linear model from scikit-learn for classification.

.. _`Linear regression`: https://scikit-learn.org/0.20/modules/generated/sklearn.linear_model.LinearRegression.html#sklearn-linear-model-linearregression
""",
  'documentation_url': 'https://lale.readthedocs.io/en/latest/modules/lale.lib.sklearn.linear_regression.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['estimator', 'regressor'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_predict': _input_predict_schema,
        'output_predict': _output_predict_schema}}

lale.docstrings.set_docstrings(LinearRegressionImpl, _combined_schemas)

LinearRegression = lale.operators.make_operator(LinearRegressionImpl, _combined_schemas)



# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sklearn.ensemble.gradient_boosting
import lale.docstrings
import lale.operators

class GradientBoostingClassifierImpl():

    def __init__(self, loss='deviance', learning_rate=0.1, n_estimators=100, subsample=1.0, criterion='friedman_mse', min_samples_split=2, min_samples_leaf=1, min_weight_fraction_leaf=0.0, max_depth=3, min_impurity_decrease=0.0, min_impurity_split=None, init=None, random_state=None, max_features=None, verbose=0, max_leaf_nodes=None, warm_start=False, presort=None, validation_fraction=0.1, n_iter_no_change=None, tol=0.0001):
        self._hyperparams = {
            'loss': loss,
            'learning_rate': learning_rate,
            'n_estimators': n_estimators,
            'subsample': subsample,
            'criterion': criterion,
            'min_samples_split': min_samples_split,
            'min_samples_leaf': min_samples_leaf,
            'min_weight_fraction_leaf': min_weight_fraction_leaf,
            'max_depth': max_depth,
            'min_impurity_decrease': min_impurity_decrease,
            'min_impurity_split': min_impurity_split,
            'init': init,
            'random_state': random_state,
            'max_features': max_features,
            'verbose': verbose,
            'max_leaf_nodes': max_leaf_nodes,
            'warm_start': warm_start,
            'presort': presort,
            'validation_fraction': validation_fraction,
            'n_iter_no_change': n_iter_no_change,
            'tol': tol}
        self._wrapped_model = sklearn.ensemble.gradient_boosting.GradientBoostingClassifier(**self._hyperparams)

    def fit(self, X, y, **fit_params):
        if fit_params is None:
            self._wrapped_model.fit(X, y)
        else:
            self._wrapped_model.fit(X, y, **fit_params)
        return self

    def predict(self, X):
        return self._wrapped_model.predict(X)

    def predict_proba(self, X):
        return self._wrapped_model.predict_proba(X)

    def decision_function(self, X):
        return self._wrapped_model.decision_function(X)

_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Gradient Boosting for classification.',
    'allOf': [{
        'type': 'object',
        'required': ['init', 'presort'],
        'relevantToOptimizer': ['loss', 'learning_rate', 'n_estimators', 'subsample', 'min_samples_split', 'min_samples_leaf', 'max_depth', 'max_features', 'presort', 'n_iter_no_change', 'tol'],
        'additionalProperties': False,
        'properties': {
            'loss': {
                'enum': ['deviance', 'exponential'],
                'default': 'deviance',
                'description': "loss function to be optimized. 'deviance' refers to"},
            'learning_rate': {
                'type': 'number',
                'minimumForOptimizer': 0.01,
                'maximumForOptimizer': 1.0,
                'distribution': 'loguniform',
                'default': 0.1,
                'description': 'learning rate shrinks the contribution of each tree by `learning_rate`.'},
            'n_estimators': {
                'type': 'integer',
                'minimumForOptimizer': 10,
                'maximumForOptimizer': 100,
                'distribution': 'uniform',
                'default': 100,
                'description': 'The number of boosting stages to perform. Gradient boosting'},
            'subsample': {
                'type': 'number',
                'minimumForOptimizer': 0.01,
                'maximumForOptimizer': 1.0,
                'distribution': 'uniform',
                'default': 1.0,
                'description': 'The fraction of samples to be used for fitting the individual base'},
            'criterion': {
                'enum': ['friedman_mse', 'mse', 'mae'],
                'default': 'friedman_mse',
                'description': 'The function to measure the quality of a split. Supported criteria'},
            'min_samples_split': {
                'anyOf': [{
                    'type': 'integer',
                    'minimumForOptimizer': 2,
                    'maximumForOptimizer': 20,
                    'distribution': 'uniform'}, {
                    'type': 'number',
                    'minimumForOptimizer': 0.01,
                    'maximumForOptimizer': 0.5}],
                'default': 2,
                'description': 'The minimum number of samples required to split an internal node:'},
            'min_samples_leaf': {
                'anyOf': [{
                    'type': 'integer',
                    'minimumForOptimizer': 1,
                    'maximumForOptimizer': 20,
                    'distribution': 'uniform'}, {
                    'type': 'number',
                    'minimumForOptimizer': 0.01,
                    'maximumForOptimizer': 0.5}],
                'default': 1,
                'description': 'The minimum number of samples required to be at a leaf node.'},
            'min_weight_fraction_leaf': {
                'type': 'number',
                'default': 0.0,
                'description': 'The minimum weighted fraction of the sum total of weights (of all'},
            'max_depth': {
                'type': 'integer',
                'minimumForOptimizer': 3,
                'maximumForOptimizer': 5,
                'default': 3,
                'description': 'maximum depth of the individual regression estimators. The maximum'},
            'min_impurity_decrease': {
                'type': 'number',
                'default': 0.0,
                'description': 'A node will be split if this split induces a decrease of the impurity'},
            'min_impurity_split': {
                'anyOf': [{
                    'type': 'number'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'Threshold for early stopping in tree growth. A node will split'},
            'init': {
                'anyOf': [{
                    'type': 'object'}, {
                    'enum': ['zero', None]}],
                'default': None,
                'description': 'An estimator object that is used to compute the initial'},
            'random_state': {
                'anyOf': [{
                    'type': 'integer'}, {
                    'type': 'object'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'If int, random_state is the seed used by the random number generator;'},
            'max_features': {
                'anyOf': [{
                    'type': 'integer',
                    'forOptimizer': False}, {
                    'type': 'number',
                    'minimum': 0.0,
                    'exclusiveMinimum': True,
                    'minimumForOptimizer': 0.0,
                    'maximumForOptimizer': 1.0,
                    'distribution': 'uniform'}, {
                    'enum': ['auto', 'sqrt', 'log2', None]}],
                'default': None,
                'description': 'The number of features to consider when looking for the best split:'},
            'verbose': {
                'type': 'integer',
                'default': 0,
                'description': 'Enable verbose output. If 1 then it prints progress and performance'},
            'max_leaf_nodes': {
                'anyOf': [{
                    'type': 'integer'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'Grow trees with ``max_leaf_nodes`` in best-first fashion.'},
            'warm_start': {
                'type': 'boolean',
                'default': False,
                'description': 'When set to ``True``, reuse the solution of the previous call to fit'},
            'presort': {
                'anyOf': [{
                    'type': 'boolean'}, {
                    'enum': ['auto']}],
                'default': 'auto',
                'description': 'Whether to presort the data to speed up the finding of best splits in'},
            'validation_fraction': {
                'type': 'number',
                'default': 0.1,
                'description': 'The proportion of training data to set aside as validation set for'},
            'n_iter_no_change': {
                'anyOf': [{
                    'type': 'integer',
                    'minimumForOptimizer': 5,
                    'maximumForOptimizer': 10}, {
                    'enum': [None]}],
                'default': None,
                'description': '``n_iter_no_change`` is used to decide if early stopping will be used'},
            'tol': {
                'type': 'number',
                'minimumForOptimizer': 1e-08,
                'maximumForOptimizer': 0.01,
                'distribution': 'loguniform',
                'default': 0.0001,
                'description': 'Tolerance for the early stopping. When the loss is not improving'},
        }}]}

_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Fit the gradient boosting model.',
    'type': 'object',
    'required': ['X', 'y'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The input samples. Internally, it will be converted to'},
        'y': {
            'anyOf': [
                {'type': 'array', 'items': {'type': 'number'}},
                {'type': 'array', 'items': {'type': 'string'}}],
            'description': 'Target values (strings or integers in classification, real numbers'},
        'sample_weight': {
            'anyOf': [{
                'type': 'array',
                'items': {
                    'type': 'number'},
            }, {
                'enum': [None]}],
            'default': None,
            'description': 'Sample weights. If None, then samples are equally weighted. Splits'},
        'monitor': {
            'type': 'object', #callable, optional
            'description': 'The monitor is called after each iteration with the current'},
    },
}
_input_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predict class for X.',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The input samples. Internally, it will be converted to'},
    },
}
_output_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'The predicted values.',
    'anyOf': [
        {'type': 'array', 'items': {'type': 'number'}},
        {'type': 'array', 'items': {'type': 'string'}}]}

_input_predict_proba_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predict class probabilities for X.',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The input samples. Internally, it will be converted to'},
    },
}
_output_predict_proba_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'The class probabilities of the input samples. The order of the',
    'type': 'array',
    'items': {
        'type': 'array',
        'items': {
            'type': 'number'},
    },
}

_input_decision_function_schema = {
  'type': 'object',
  'required': ['X'],
  'additionalProperties': False,
  'properties': {
    'X': {
      'description': 'Features; the outer array is over samples.',
      'type': 'array',
      'items': {'type': 'array', 'items': {'type': 'number'}}}}}

_output_decision_function_schema = {
    'description': 'Confidence scores for samples for each class in the model.',
    'anyOf': [
    {   'description': 'In the multi-way case, score per (sample, class) combination.',
        'type': 'array',
        'items': {'type': 'array', 'items': {'type': 'number'}}},
    {   'description': 'In the binary case, score for `self._classes[1]`.',
        'type': 'array',
        'items': {'type': 'number'}}]}

_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': """`Gradient boosting classifier`_ random forest from scikit-learn.

.. _`Gradient boosting classifier`: https://scikit-learn.org/0.20/modules/generated/sklearn.ensemble.GradientBoostingClassifier.html#sklearn-ensemble-gradientboostingclassifier
""",
    'documentation_url': 'https://lale.readthedocs.io/en/latest/modules/lale.lib.sklearn.gradient_boosting_classifier.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['estimator', 'classifier'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_predict': _input_predict_schema,
        'output_predict': _output_predict_schema,
        'input_predict_proba': _input_predict_proba_schema,
        'output_predict_proba': _output_predict_proba_schema,
        'input_decision_function': _input_decision_function_schema,
        'output_decision_function': _output_decision_function_schema,
}}

lale.docstrings.set_docstrings(GradientBoostingClassifierImpl, _combined_schemas)

GradientBoostingClassifier = lale.operators.make_operator(GradientBoostingClassifierImpl, _combined_schemas)



# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sklearn.base import BaseEstimator
import lale.docstrings

class XGBClassifierImpl(BaseEstimator):
    def __init__(self, max_depth=3, learning_rate=0.1, n_estimators=100, verbosity=1, 
                objective='binary:logistic', booster='gbtree', tree_method='auto', n_jobs=1, 
                nthread=None, gamma=0, min_child_weight=1, max_delta_step=0, subsample=1, 
                colsample_bytree=1, colsample_bylevel=1, colsample_bynode=1, reg_alpha=0, 
                reg_lambda=1, scale_pos_weight=1, base_score=0.5, random_state=0, 
                seed=None, missing=None, silent=None):
        self.max_depth = max_depth
        self.learning_rate = learning_rate
        self.n_estimators = n_estimators
        self.verbosity = verbosity
        self.objective = objective
        self.booster = booster
        self.tree_method = tree_method
        self.n_jobs = n_jobs
        self.nthread = nthread
        self.gamma = gamma
        self.min_child_weight = min_child_weight
        self.max_delta_step = max_delta_step
        self.subsample = subsample
        self.colsample_bytree = colsample_bytree
        self.colsample_bylevel = colsample_bylevel
        self.colsample_bynode = colsample_bynode
        self.reg_alpha = reg_alpha
        self.reg_lambda = reg_lambda
        self.scale_pos_weight = scale_pos_weight
        self.base_score = base_score
        self.random_state = random_state
        self.seed = seed
        self.missing = missing
        self.silent = silent

    def fit(self, X, y, **fit_params):
        result = XGBClassifierImpl(self.max_depth, self.learning_rate, self.n_estimators, 
                self.verbosity, self.objective, self.booster, self.tree_method, self.n_jobs, 
                self.nthread, self.gamma, self.min_child_weight, self.max_delta_step, self.subsample, 
                self.colsample_bytree, self.colsample_bylevel, self.colsample_bynode, self.reg_alpha, 
                self.reg_lambda, self.scale_pos_weight, self.base_score, self.random_state, 
                self.seed, self.missing, self.silent)
        result._wrapped_model = XGBoostClassifier(
                    **self.get_params())
        if fit_params is None:
            result._wrapped_model.fit(X, y)
        else:
            result._wrapped_model.fit(X, y, **fit_params)
        return result

    def predict(self, X):
        return self._wrapped_model.predict(X)

    def predict_proba(self, X):
        return self._wrapped_model.predict_proba(X)

from xgboost import XGBClassifier as XGBoostClassifier
import lale.operators

_hyperparams_schema = {
  '$schema': 'http://json-schema.org/draft-04/schema#',
  'description': 'Hyperparameter schema for a Lale wrapper for XGBoost.',
  'allOf': [
    { 'description':
        'This first sub-object lists all constructor arguments with their '
        'types, one at a time, omitting cross-argument constraints.',
      'type': 'object',
      'additionalProperties': False,
      'required': ['max_depth', 'learning_rate','n_estimators',
      'verbosity', 'objective', 'booster', 'tree_method', 'n_jobs', 'gamma','min_child_weight',
      'max_delta_step', 'subsample', 'colsample_bytree', 'colsample_bylevel',
      'colsample_bynode', 'reg_alpha', 'reg_lambda', 'scale_pos_weight',
      'base_score', 'random_state', 'missing'],
      'relevantToOptimizer': ['max_depth','learning_rate','n_estimators','booster', 'min_child_weight',
      'subsample', 'colsample_bytree', 'colsample_bylevel', 'reg_alpha', 'reg_lambda'],
      #'objective', 'booster', ],
      'properties': {
        'max_depth': {
          'description': 'Maximum tree depth for base learners.',
          'type': 'integer',
          'default': 3,
          'minimum': 0,
          'distribution':'uniform',
          'minimumForOptimizer': 1,
          'maximumForOptimizer': 20},
        'learning_rate': {
          'description': 'Boosting learning rate (xgb’s “eta”)',
          'type': 'number',
          'default': 0.1,
          'distribution':'loguniform',
          'minimumForOptimizer': 0.01,
          'maximumForOptimizer': 1},
        'n_estimators': {
          'description': 'Number of trees to fit.',
          'type': 'integer',
          'default': 100,
          'minimumForOptimizer': 10,
          'maximumForOptimizer': 1500},
        'verbosity': {
          'description': 'The degree of verbosity.',
          'type': 'integer',
          'default': 1,
          'minimum': 0,
          'maximum': 3},
        'objective': {
          'description': 'Specify the learning task and the corresponding '
           'learning objective or a custom objective function to be used.'
           ' string or callable.',
          'enum': ['binary:logistic', 'binary:logitraw', 'binary:hinge', 'multi:softprob','multi:softmax'],
          'default': 'binary:logistic'},
        'booster': {
          'description':
            'Specify which booster to use.',
          'enum': ['gbtree', 'gblinear', 'dart'],
          'default': 'gbtree'
        },
        'tree_method':{
          'description': """Specify which tree method to use. 
Default to auto. If this parameter is set to default, XGBoost will choose the most conservative option available.
Refer to https://xgboost.readthedocs.io/en/latest/parameter.html. """,
          'enum':['auto', 'exact','approx', 'hist', 'gpu_hist'],
          'default':'auto'},
        'n_jobs': {
            'type': 'integer',
            'description': 'Number of parallel threads used to run xgboost.  (replaces ``nthread``)',
            'default': 1
        },
        'nthread': {
            'anyOf': [
            {'type':'integer'},
            {'enum':[None]}],
            'default': None,
            'description': 'Number of parallel threads used to run xgboost.  Deprecated, please use n_jobs'},
        'gamma': {
            'type': 'number',
            'description': 'Minimum loss reduction required to make a further partition on a leaf node of the tree.',
            'default': 0,
            'minimum': 0
        },
        'min_child_weight': {
            'type': 'integer',
            'description': 'Minimum sum of instance weight(hessian) needed in a child.',
            'default': 1,
            'distribution': 'uniform',
            'minimumForOptimizer' : 1,
            'maximumForOptimizer': 20
        },
        'max_delta_step': {
            'type': 'integer',
            'description': "Maximum delta step we allow each tree's weight estimation to be.",
            'default': 0,
        },
        'subsample': {
            'type': 'number',
            'description': 'Subsample ratio of the training instance.',
            'default': 1,
            'minimum': 0,
            'exclusiveMinimum': True,
            'distribution': 'uniform',
            'minimumForOptimizer' : 0.01,
            'maximumForOptimizer': 1.0
        },
        'colsample_bytree': {
            'type': 'number',
            'description': 'Subsample ratio of columns when constructing each tree.',
            'default': 1,
            'minimum': 0,
            'exclusiveMinimum': True,
            'maximum': 1,
            'distribution': 'uniform',
            'minimumForOptimizer' : 0.1,
            'maximumForOptimizer': 1.0
        },
        'colsample_bylevel': {
            'type': 'number',
            'description': 'Subsample ratio of columns for each split, in each level.',
            'default': 1,
            'minimum': 0,
            'exclusiveMinimum': True,
            'maximum': 1,
            'distribution': 'uniform',
            'minimumForOptimizer' : 0.1,
            'maximumForOptimizer': 1.0
        },
        'colsample_bynode': {
            'type': 'number',
            'description': 'Subsample ratio of columns for each split.',
            'default': 1,
            'minimum': 0,
            'exclusiveMinimum': True,
            'maximum': 1
        },
        'reg_alpha': {
            'type': 'number',
            'description': 'L1 regularization term on weights',
            'default': 0,
            'distribution': 'uniform',
            'minimumForOptimizer': 0.0,
            'maximumForOptimizer': 1.0
        },
        'reg_lambda': {
            'type': 'number',
            'description': 'L2 regularization term on weights',
            'default': 1,
            'distribution': 'uniform',
            'minimumForOptimizer': 0.1,
            'maximumForOptimizer': 1.0
        },
        'scale_pos_weight': {
            'type': 'number',
            'description': 'Balancing of positive and negative weights.',
            'default': 1
        },
        'base_score': {
            'type': 'number',
            'description': 'The initial prediction score of all instances, global bias.',
            'default': 0.5
        },
        'random_state': {
            'type': 'integer',
            'description': 'Random number seed.  (replaces seed)',
            'default': 0
        },
        'missing': {
            'anyOf': [{
                'type': 'number',
            }, {
                'enum': [None],
            }],
            'default': None,
            'description': 'Value in the data which needs to be present as a missing value. If'
            ' If None, defaults to np.nan.'
        },
        'silent':{
            'anyOf': [{
                'type': 'boolean',
            }, {
                'enum': [None],
            }],
            'default': None,
            'description': 'deprecated and replaced with verbosity, but adding to be backward compatible. '
        },
        'seed': {
            'default': None,
            'description': 'deprecated and replaced with random_state, but adding to be backward compatible. '
        }
        },
    }],
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Fit gradient boosting classifier',
    'type': 'object',
    'required': ['X', 'y'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'Feature matrix',
        },
        'y': {
            'anyOf': [
                {'type': 'array', 'items': {'type': 'number'}},
                {'type': 'array', 'items': {'type': 'string'}}],
            'description': 'Labels',
        },
        'sample_weight': {
            'anyOf': [{
                'type': 'array',
                'items': {
                    'type': 'number'},
            }, {
                'enum': [None]}],
            'description': 'Weight for each instance',
            'default': None
        },
        'eval_set': {
            'anyOf': [{
                'type': 'array',
            }, {
                'enum': [None],
            }],
            'default': None,
            'description': 'A list of (X, y) pairs to use as a validation set for',
        },
        'sample_weight_eval_set': {
            'anyOf': [{
                'type': 'array',
            }, {
                'enum': [None],
            }],
            'default': None,
            'description': 'A list of the form [L_1, L_2, ..., L_n], where each L_i is a list of',
        },
        'eval_metric': {
            'anyOf': [{
                'type': 'array',
                'items': {'type':'string'}}, {
                'type': 'string'},{
                'enum': [None]},{
                'type': 'object'}],
            'default': None,
            'description': 'If a str, should be a built-in evaluation metric to use. See',
        },
        'early_stopping_rounds': {
            'anyOf': [{
                'type': 'integer',
            }, {
                'enum': [None],
            }],
            'default': None,
            'description': 'Activates early stopping. Validation error needs to decrease at',
        },
        'verbose': {
            'type': 'boolean',
            'description': 'If `verbose` and an evaluation set is used, writes the evaluation',
            'default': True
        },
        'xgb_model': {
            'anyOf':[{
                'type': 'string'},{
                'enum': [None]}],
            'description': "file name of stored xgb model or 'Booster' instance Xgb model to be",
            'default': None
        },
        'callbacks': {
            'anyOf': [{
                'type': 'array',
                'items': {'type':'object'}}, {
                'enum': [None]}],
            'default': None,
            'description': 'List of callback functions that are applied at each iteration. '}
    }
}

_input_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predict with `data`.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'}},
            'description': 'The dmatrix storing the input.'},
        'output_margin': {
            'type': 'boolean',
            'default': False,
            'description': 'Whether to output the raw untransformed margin value.'},
        'ntree_limit': {
            'anyOf':[
            {'type': 'integer'},
            {'enum': [None]}],
            'description': 'Limit number of trees in the prediction; defaults to best_ntree_limit if defined'},
        'validate_features': {
            'type': 'boolean',
            'default': True,
            'description': "When this is True, validate that the Booster's and data's feature_names are identical.",
        }}}

_output_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predicted class label per sample.',
    'anyOf': [
        {'type': 'array', 'items': {'type': 'number'}},
        {'type': 'array', 'items': {'type': 'string'}}]}

_input_predict_proba_schema = {
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'}}}}}

_output_predict_proba_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Probability of the sample for each class in the model.',
    'type': 'array',
    'items': {
        'type': 'array',
        'items': {
            'type': 'number'}}}

_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Combined schema for expected data and hyperparameters.',
    'documentation_url': 'https://xgboost.readthedocs.io/en/latest/python/python_api.html#module-xgboost.sklearn',
    'tags': {
        'pre': [],
        'op': ['estimator', 'classifier'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_predict': _input_predict_schema,
        'output_predict': _output_predict_schema,
        'input_predict_proba': _input_predict_proba_schema,
        'output_predict_proba': _output_predict_proba_schema}}

lale.docstrings.set_docstrings(XGBClassifierImpl, _combined_schemas)

XGBClassifier = lale.operators.make_operator(XGBClassifierImpl, _combined_schemas)




from sklearn.preprocessing.data import RobustScaler as Op
import lale.helpers
import lale.operators
import lale.docstrings
from numpy import nan, inf

class RobustScalerImpl():

    def __init__(self, with_centering=True, with_scaling=True, quantile_range=(25.0, 75.0), copy=True):
        self._hyperparams = {
            'with_centering': with_centering,
            'with_scaling': with_scaling,
            'quantile_range': quantile_range,
            'copy': copy}
        self._wrapped_model = Op(**self._hyperparams)

    def fit(self, X, y=None):
        if (y is not None):
            self._wrapped_model.fit(X, y)
        else:
            self._wrapped_model.fit(X)
        return self

    def transform(self, X):
        return self._wrapped_model.transform(X)
_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'inherited docstring for RobustScaler    Scale features using statistics that are robust to outliers.',
    'allOf': [{
        'type': 'object',
        'required': ['with_centering', 'with_scaling', 'quantile_range', 'copy'],
        'relevantToOptimizer': ['with_centering', 'with_scaling', 'copy'],
        'additionalProperties': False,
        'properties': {
            'with_centering': {
                'type': 'boolean',
                'default': True,
                'description': 'If True, center the data before scaling'},
            'with_scaling': {
                'type': 'boolean',
                'default': True,
                'description': 'If True, scale the data to interquartile range.'},
            'quantile_range': {
                'XXX TODO XXX': 'tuple (q_min, q_max), 0.0 < q_min < q_max < 100.0',
                'description': 'Default: (25.0, 75.0) = (1st quantile, 3rd quantile) = IQR Quantile range used to calculate ``scale_``',
                'type': 'array',
                'laleType': 'tuple',
                'default': (25.0, 75.0)},
            'copy': {
                'XXX TODO XXX': 'boolean, optional, default is True',
                'description': 'If False, try to avoid a copy and do inplace scaling instead',
                'type': 'boolean',
                'default': True},
        }}],
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Compute the median and quantiles to be used for scaling.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The data used to compute the median and quantiles used for later scaling along the features axis.'},
    },
}
_input_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Center and scale the data.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'laleType': 'Any',
                'XXX TODO XXX': 'item type'},
            'XXX TODO XXX': '{array-like, sparse matrix}',
            'description': 'The data used to scale along the specified axis.'},
    },
}
_output_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Center and scale the data.',
    'laleType': 'Any',
}
_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Combined schema for expected data and hyperparameters.',
    'documentation_url': 'https://scikit-learn.org/0.20/modules/generated/sklearn.preprocessing.RobustScaler#sklearn-preprocessing-robustscaler',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['transformer'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_transform': _input_transform_schema,
        'output_transform': _output_transform_schema},
}
lale.docstrings.set_docstrings(RobustScalerImpl, _combined_schemas)
RobustScaler = lale.operators.make_operator(RobustScalerImpl, _combined_schemas)




# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sklearn.impute import MissingIndicator as SKLModel
import lale.docstrings
import lale.operators
from numpy import nan

class MissingIndicatorImpl():

    def __init__(self, missing_values='nan', features='missing-only', sparse='auto', error_on_new=True):
        self._hyperparams = {
            'missing_values': missing_values,
            'features': features,
            'sparse': sparse,
            'error_on_new': error_on_new}

    def fit(self, X, y=None):
        self._wrapped_model = SKLModel(**self._hyperparams)
        if (y is not None):
            self._wrapped_model.fit(X, y)
        else:
            self._wrapped_model.fit(X)
        return self

    def transform(self, X):
        return self._wrapped_model.transform(X)
_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'inherited docstring for MissingIndicator    Binary indicators for missing values.',
    'allOf': [{
        'type': 'object',
        'required': ['missing_values', 'features', 'sparse', 'error_on_new'],
        'relevantToOptimizer': [],
        'additionalProperties': False,
        'properties': {
            'missing_values': {
                'anyOf': [{'type': 'number'},
                          {'type': 'string'},
                          {'enum': [nan]},
                          {'enum': [None]}],
                'description': 'The placeholder for the missing values.',
                'type': 'number',
                'default': nan},
            'features': {
                'type': 'string',
                'default': 'missing-only',
                'description': 'Whether the imputer mask should represent all or a subset of'},
            'sparse': {
                'anyOf': [{'type': 'boolean'},
                          {'enum': ['auto']}],
                'description': 'Whether the imputer mask format should be sparse or dense.',
                'default': 'auto'},
            'error_on_new': {
                'type': 'boolean',
                'default': True,
                'description': 'If True (default), transform will raise an error when there are'},
        }}, {
        'description': 'error_on_new, only when features="missing-only"',
        'anyOf': [{
            'type': 'object',
            'properties': {
                'error_on_new': {
                    'enum': [True]},
            }}, {
            'type': 'object',
            'properties': {
                'features': {
                    'enum': ['missing-only']},
            }}]}],
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Fit the transformer on X.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'Input data, where ``n_samples`` is the number of samples and'},
    },
}
_input_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Generate missing values indicator for X.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The input data to complete.'},
    },
}
_output_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'The missing indicator for input data.',
    'type': 'array',
    'items': {'type': 'array', 
        'items':{
            'type': 'boolean'}}
}
_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': """`Missing values indicator`_ transformer from scikit-learn.

.. _`Missing values indicator`: https://scikit-learn.org/0.20/modules/generated/sklearn.impute.MissingIndicator.html#sklearn-impute-missingindicator
""",
    'documentation_url': 'https://lale.readthedocs.io/en/latest/modules/lale.lib.sklearn.missing_indicator.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['transformer'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_transform': _input_transform_schema,
        'output_transform': _output_transform_schema}}

lale.docstrings.set_docstrings(MissingIndicatorImpl, _combined_schemas)

MissingIndicator = lale.operators.make_operator(MissingIndicatorImpl, _combined_schemas)



# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sklearn.ensemble.weight_boosting import AdaBoostRegressor as SKLModel
import lale.docstrings
import lale.operators

class AdaBoostRegressorImpl():

    def __init__(self, base_estimator=None, n_estimators=50, learning_rate=1.0, loss='linear', random_state=None):
        if isinstance(base_estimator, lale.operators.Operator):
            if isinstance(base_estimator, lale.operators.IndividualOp):
                base_estimator = base_estimator._impl_instance()._wrapped_model
            else:
                raise ValueError("If base_estimator is a Lale operator, it needs to be an individual operator. ")
        self._hyperparams = {
            'base_estimator': base_estimator,
            'n_estimators': n_estimators,
            'learning_rate': learning_rate,
            'loss': loss,
            'random_state': random_state}
        self._wrapped_model = SKLModel(**self._hyperparams)

    def fit(self, X, y=None):
        if (y is not None):
            self._wrapped_model.fit(X, y)
        else:
            self._wrapped_model.fit(X)
        return self

    def predict(self, X):
        return self._wrapped_model.predict(X)
_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'inherited docstring for AdaBoostRegressor    An AdaBoost regressor.',
    'allOf': [{
        'type': 'object',
        'required': ['base_estimator', 'n_estimators', 'learning_rate', 'loss', 'random_state'],
        'relevantToOptimizer': ['n_estimators', 'learning_rate', 'loss'],
        'additionalProperties': False,
        'properties': {
            'base_estimator': {
                'anyOf': [{
                    'laleType' : 'operator'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'The base estimator from which the boosted ensemble is built.'},
            'n_estimators': {
                'type': 'integer',
                'minimumForOptimizer': 50,
                'maximumForOptimizer': 500,
                'distribution': 'uniform',
                'default': 50,
                'description': 'The maximum number of estimators at which boosting is terminated.'},
            'learning_rate': {
                'type': 'number',
                'minimumForOptimizer': 0.01,
                'maximumForOptimizer': 1.0,
                'distribution': 'loguniform',
                'default': 1.0,
                'description': 'Learning rate shrinks the contribution of each regressor by'},
            'loss': {
                'enum': ['linear', 'square', 'exponential'],
                'default': 'linear',
                'description': 'The loss function to use when updating the weights after each'},
            'random_state': {
                'anyOf': [{
                    'type': 'integer'}, {
                    'type': 'object'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'If int, random_state is the seed used by the random number generator;'},
        }}],
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Build a boosted regressor from the training set (X, y).',
    'required': ['X', 'y'],
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The training input samples. Sparse matrix can be CSC, CSR, COO,'},
        'y': {
            'type': 'array',
            'items': {
                'type': 'number'},
            'description': 'The target values (real numbers).'},
        'sample_weight': {
            'anyOf': [{
                'type': 'array',
                'items': {
                    'type': 'number'},
            }, {
                'enum': [None]}],
            'default': None,
            'description': 'Sample weights. If None, the sample weights are initialized to'},
    },
}
_input_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predict regression value for X.',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The training input samples. Sparse matrix can be CSC, CSR, COO,'},
    },
}
_output_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'The predicted regression values.',
    'type': 'array',
    'items': {
        'type': 'number'},
}
_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': """`AdaBoost regressor`_ from scikit-learn for boosting ensemble.

.. _`AdaBoost regressor`: https://scikit-learn.org/0.20/modules/generated/sklearn.ensemble.AdaBoostRegressor.html#sklearn-ensemble-adaboostregressor
""",
    'documentation_url': 'https://lale.readthedocs.io/en/latest/modules/lale.lib.sklearn.ada_boost_regressor.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['estimator', 'regressor'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_predict': _input_predict_schema,
        'output_predict': _output_predict_schema}}

lale.docstrings.set_docstrings(AdaBoostRegressorImpl, _combined_schemas)

AdaBoostRegressor = lale.operators.make_operator(AdaBoostRegressorImpl, _combined_schemas)




# Copyright 2020 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import lale.docstrings
import lale.operators
import autoai_libs.transformers.exportable

class CompressStringsImpl():
    def __init__(self, compress_type, dtypes_list, misslist_list, missing_values_reference_list, activate_flag):
        self._hyperparams = {
            'compress_type': compress_type,
            'dtypes_list': dtypes_list,
            'misslist_list': misslist_list,
            'missing_values_reference_list': missing_values_reference_list,
            'activate_flag': activate_flag}
        self._wrapped_model = autoai_libs.transformers.exportable.CompressStrings(**self._hyperparams)

    def fit(self, X, y=None):
        self._wrapped_model.fit(X, y)
        return self

    def transform(self, X):
        return self._wrapped_model.transform(X)

_hyperparams_schema = {
    'allOf': [{
        'description': 'This first object lists all constructor arguments with their types, but omits constraints for conditional hyperparameters.',
        'type': 'object',
        'additionalProperties': False,
        'required': ['compress_type', 'dtypes_list', 'misslist_list', 'missing_values_reference_list', 'activate_flag'],
        'relevantToOptimizer': ['compress_type', 'activate_flag'],
        'properties': {
            'compress_type': {
                'description': 'Type of string compression: `string` for removing spaces from a string and `hash` for creating an int hash, used when there are columns with strings and cat_imp_strategy=`most_frequent`.',
                'enum': ['string', 'hash'],
                'default': 'string'},
            'dtypes_list': {
                'anyOf': [
                {   'description': 'Strings that denote the type of each column of the input numpy array X.',
                    'type': 'array',
                    'items': {
                        'enum': ['char_str', 'int_str', 'float_str', 'float_num', 'float_int_num', 'int_num', 'boolean', 'Unknown']}},
                {   'description': 'If None, the column types are discovered.',
                    'enum': [None]}],
                'default': None},
            'misslist_list': {
                'anyOf': [
                {   'description': 'List containing lists of missing values of each column of the input numpy array X.',
                    'type': 'array',
                    'items': {'type': 'array', 'items': {'laleType': 'Any'}}},
                {   'description': 'If None, the missing values of each column are discovered.',
                    'enum': [None]}],
                'default': None},
            'missing_values_reference_list': {
                'anyOf': [
                {   'description': 'Reference list of missing values in the input numpy array X.',

                    'type': 'array',
                    'items': {'laleType': 'Any'}},
                {   'description': 'If None, the missing values of each column are discovered.',
                    'enum': [None]}],
                'default': None},
            'activate_flag': {
                'description': 'If False, transform(X) outputs the input numpy array X unmodified.',
                'type': 'boolean',
                'default': True}}}]}

_input_fit_schema = {
    'type': 'object',
    'required': ['X'],
    'additionalProperties': False,
    'properties': {
        'X': {#Handles 1-D arrays as well
            'anyOf': [
                {'type': 'array', 'items': {'laleType': 'Any'}},
                {'type': 'array', 'items': {'type': 'array', 'items': {'laleType': 'Any'}}}]},
        'y': {
            'laleType': 'Any'}}}

_input_transform_schema = {
    'type': 'object',
    'required': ['X'],
    'additionalProperties': False,
    'properties': {
        'X': {#Handles 1-D arrays as well
            'anyOf': [
                {'type': 'array', 'items': {'laleType': 'Any'}},
                {'type': 'array', 'items': {'type': 'array', 'items': {'laleType': 'Any'}}}]}}}

_output_transform_schema = {
    'description': 'Features; the outer array is over samples.',
    'anyOf': [
        {'type': 'array', 'items': {'laleType': 'Any'}},
        {'type': 'array', 'items': {'type': 'array', 'items': {'laleType': 'Any'}}}]}

_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': """Operator from `autoai_libs`_. Removes spaces and special characters from string columns of a numpy array.

.. _`autoai_libs`: https://pypi.org/project/autoai-libs""",
    'documentation_url': 'https://lale.readthedocs.io/en/latest/modules/lale.lib.autoai_libs.compress_strings.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['transformer'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_transform': _input_transform_schema,
        'output_transform': _output_transform_schema}}

lale.docstrings.set_docstrings(CompressStringsImpl, _combined_schemas)

CompressStrings = lale.operators.make_operator(CompressStringsImpl, _combined_schemas)



# Copyright 2020 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import lale.docstrings
import lale.operators
import autoai_libs.transformers.exportable

class NumpyReplaceUnknownValuesImpl():
    def __init__(self, known_values_list=None, filling_values=None, missing_values_reference_list=None, 
                filling_values_list=None):
        self._hyperparams = {
            'known_values_list': known_values_list,
            'filling_values': filling_values,
            'missing_values_reference_list': missing_values_reference_list,
            'filling_values_list': filling_values_list}
        self._wrapped_model = autoai_libs.transformers.exportable.NumpyReplaceUnknownValues(**self._hyperparams)

    def fit(self, X, y=None):
        self._wrapped_model.fit(X, y)
        return self

    def transform(self, X):
        return self._wrapped_model.transform(X)

_hyperparams_schema = {
    'allOf': [{
        'description': 'This first object lists all constructor arguments with their types, but omits constraints for conditional hyperparameters.',
        'type': 'object',
        'additionalProperties': False,
        'required': ['known_values_list', 'filling_values', 'missing_values_reference_list', 'filling_values_list'],
        'relevantToOptimizer': [],
        'properties': {
            'known_values_list': {
                'description': 'Reference list of lists of known values for each column.',
                'anyOf': [
                {   'type': 'array',
                    'items': {'laleType': 'Any'}},
                {   'enum': [None]}],
                'default': None},
            'filling_values': {
                'description': 'Special value assigned to unknown values.',
                'laleType': 'Any',
                'default': None},
            'missing_values_reference_list': {
                'description': 'Reference list of missing values.',
                'anyOf': [
                {   'type': 'array',
                    'items': {'laleType': 'Any'}},
                {   'enum': [None]}],
                'default': None},
            'filling_values_list': {
                'description': 'list of special value assigned to unknown values.',
                'anyOf': [
                {   'type': 'array',
                    'items': {'laleType': 'Any'}},
                {   'enum': [None]}],
                'default': None}}}]}

_input_fit_schema = {
    'type': 'object',
    'required': ['X'],
    'additionalProperties': False,
    'properties': {
        'X': {#Handles 1-D arrays as well
            'anyOf': [
                {'type': 'array', 'items': {'laleType': 'Any'}},
                {'type': 'array', 'items': {'type': 'array', 'items': {'laleType': 'Any'}}}]},
        'y': {
            'laleType': 'Any'}}}

_input_transform_schema = {
    'type': 'object',
    'required': ['X'],
    'additionalProperties': False,
    'properties': {
        'X': {#Handles 1-D arrays as well
            'anyOf': [
                {'type': 'array', 'items': {'laleType': 'Any'}},
                {'type': 'array', 'items': {'type': 'array', 'items': {'laleType': 'Any'}}}]}}}

_output_transform_schema = {
    'description': 'Features; the outer array is over samples.',
    'anyOf': [
        {'type': 'array', 'items': {'laleType': 'Any'}},
        {'type': 'array', 'items': {'type': 'array', 'items': {'laleType': 'Any'}}}]}

_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': """Operator from `autoai_libs`_. Given a numpy array and a reference list of known values for each column, replaces values that are not part of a reference list with a special value (typically np.nan). This is typically used to remove labels for columns in a test dataset that have not been seen in the corresponding columns of the training dataset.

.. _`autoai_libs`: https://pypi.org/project/autoai-libs""",
    'documentation_url': 'https://lale.readthedocs.io/en/latest/modules/lale.lib.autoai_libs.numpy_replace_unknown_values.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['transformer'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_transform': _input_transform_schema,
        'output_transform': _output_transform_schema}}

lale.docstrings.set_docstrings(NumpyReplaceUnknownValuesImpl, _combined_schemas)

NumpyReplaceUnknownValues = lale.operators.make_operator(NumpyReplaceUnknownValuesImpl, _combined_schemas)
